#pragma once

#include "registers.hpp"
#include "system.hpp"

#include "debug.hpp"
#include "syscfg.hpp"
#include "pwr.hpp"
#include "flash.hpp"
#include "rcc.hpp"
#include "backupSRAM.hpp"
#include "gpio.hpp"
#include "hsem.hpp"
#include "rtc.hpp"

#include "exti.hpp"
#include "vrefbuf.hpp"

#include "dma2d.hpp"
#include "mdma.hpp"
#include "dmamux.hpp"
#include "dma_1.hpp"
#include "dma_2.hpp"

#include "adc_1.hpp"
#include "adc_2.hpp"
#include "adc_3.hpp"

#include "timer_1.hpp"
#include "timer_2.hpp"
#include "timer_3.hpp"
#include "timer_12.hpp"
#include "timer_13.hpp"
#include "timer_15.hpp"
#include "timer_16.hpp"
#include "timer_17.hpp"

#include "uart_5.hpp"
#include "uart_7.hpp"

#include "fmc.hpp"
#include "ltdc.hpp"

#include "i2c_1.hpp"

#include "sdmmc_1.hpp"
#include "sdmmc_2.hpp"





uint32 __startup__();













class STM32H753BIT6
{
	public:
		
		
		
		
		
		
		
	private:
		
		typedef struct
		{
			uint32 vector[NVIC::c_numberOfInterrupts];
		}s_interruptTable;
		
		static const s_interruptTable c_interruptTable;
		
		Debug m_debug;
		SYSCFG m_syscfg;
		Pwr m_pwr;
		Flash m_flash;
		RCC m_rcc;
		BackupSRAM m_backupSRAM;
		GPIO m_gpio;
		HSEM m_hsem[HSEM::c_number];
		RTC m_rtc;
		
		EXTI m_exti;
		VREFBUF m_vrefbuf;
		
		DMA2D m_dma2d;
		MDMA m_mdma;
		DMAMUX m_dmamux;
		DMA_1 m_dma_1[DMA_1::c_channel];
		DMA_2 m_dma_2[DMA_2::c_channel];
		
		ADC_1 m_adc_1;
		ADC_2 m_adc_2;
		ADC_3 m_adc_3;
		
		Timer_1 m_timer_1;
		Timer_2 m_timer_2;
		Timer_3 m_timer_3;
		Timer_12 m_timer_12;
		Timer_13 m_timer_13;
		Timer_15 m_timer_15;
		Timer_16 m_timer_16;
		Timer_17 m_timer_17;
		
		UART_5 m_uart_5;
		UART_7 m_uart_7;
		
		FMC m_fmc;
		LTDC m_ltdc;
		
		I2C_1 m_i2c_1;
		
		SDMMC_1 m_sdmmc_1;
		SDMMC_2 m_sdmmc_2;
		
		static void copy_flashToRAM(const uint32* const src, uint32* const dest, uint32 sizeInBytes);
		static void zeroOutRAM(uint32* const startAddress, uint32 sizeInBytes);
		
		friend void EXCEPTION_RESET();
		
		
		
		
		
	public:
		
		STM32H753BIT6();
		
		static STM32H753BIT6& get();
		feedback startup();
		
		
		Debug& get_debug();
		Pwr& get_pwr();
		Flash& get_flash();
		RCC& get_rcc();
		BackupSRAM& get_backupSRAM();
		GPIO& get_gpio();
		HSEM& get_hsem(uint8 number);
		RTC& get_rtc();
		
		EXTI& get_exti();
		VREFBUF& get_vrefbuf();
		
		DMA2D& get_dma2d();
		MDMA& get_mdma();
		DMAMUX& get_dmamux();
		DMA_1& get_dma_1(uint8 channel);
		DMA_2& get_dma_2(uint8 channel);
		
		ADC_1& get_adc_1();
		ADC_2& get_adc_2();
		ADC_3& get_adc_3();
		
		Timer_1& get_timer_1();
		Timer_2& get_timer_2();
		Timer_3& get_timer_3();
		Timer_12& get_timer_12();
		Timer_13& get_timer_13();
		Timer_15& get_timer_15();
		Timer_16& get_timer_16();
		Timer_17& get_timer_17();
		
		UART_5& get_uart_5();
		UART_7& get_uart_7();
		
		FMC& get_fmc();
		LTDC& get_ltdc();
		
		I2C_1& get_i2c_1();
		
		SDMMC_1& get_sdmmc_1();
		SDMMC_2& get_sdmmc_2();
};









extern CODE_ISR void ISR_WWDG();
extern CODE_ISR void ISR_PVD_PVM();
extern CODE_ISR void ISR_RTC_TAMPER_CSSLSE();
extern CODE_ISR void ISR_RTC_WAKEUP();
extern CODE_ISR void ISR_FLASH();
extern CODE_ISR void ISR_RCC();
extern CODE_ISR void ISR_EXTI_0();
extern CODE_ISR void ISR_EXTI_1();
extern CODE_ISR void ISR_EXTI_2();
extern CODE_ISR void ISR_EXTI_3();
extern CODE_ISR void ISR_EXTI_4();
extern CODE_ISR void ISR_DMA_1_CH0();
extern CODE_ISR void ISR_DMA_1_CH1();
extern CODE_ISR void ISR_DMA_1_CH2();
extern CODE_ISR void ISR_DMA_1_CH3();
extern CODE_ISR void ISR_DMA_1_CH4();
extern CODE_ISR void ISR_DMA_1_CH5();
extern CODE_ISR void ISR_DMA_1_CH6();
extern CODE_ISR void ISR_ADC_1_2();
extern CODE_ISR void ISR_FDCAN_1_INT0();
extern CODE_ISR void ISR_FDCAN_2_INT0();
extern CODE_ISR void ISR_FDCAN_1_INT1();
extern CODE_ISR void ISR_FDCAN_2_INT1();
extern CODE_ISR void ISR_EXTI_9_TO_5();
extern CODE_ISR void ISR_TIMER_1_BREAK();
extern CODE_ISR void ISR_TIMER_1_UPDATE();
extern CODE_ISR void ISR_TIMER_1_TRIGGER_COMMUTATION();
extern CODE_ISR void ISR_TIMER_1_CAPTURE_COMPARE();
extern CODE_ISR void ISR_TIMER_2();
extern CODE_ISR void ISR_TIMER_3();
extern CODE_ISR void ISR_TIMER_4();
extern CODE_ISR void ISR_I2C_1_EVENT();
extern CODE_ISR void ISR_I2C_1_ERROR();
extern CODE_ISR void ISR_I2C_2_EVENT();
extern CODE_ISR void ISR_I2C_2_ERROR();
extern CODE_ISR void ISR_SPI_1();
extern CODE_ISR void ISR_SPI_2();
extern CODE_ISR void ISR_USART_1();
extern CODE_ISR void ISR_USART_2();
extern CODE_ISR void ISR_USART_3();
extern CODE_ISR void ISR_EXTI_15_TO_10();
extern CODE_ISR void ISR_RTC_ALARM_EXTI();
extern CODE_ISR void ISR_TIMER_8_BREAK_AND_TIMER_12();
extern CODE_ISR void ISR_TIMER_8_UPDATE_AND_TIMER_13();
extern CODE_ISR void ISR_TIMER_8_TRIGGER_COMMUTATION_AND_TIMER_14();
extern CODE_ISR void ISR_TIMER_8_CAPTURE_COMPARE();
extern CODE_ISR void ISR_DMA_1_CH7();
extern CODE_ISR void ISR_FMC();
extern CODE_ISR void ISR_SDMMC_1();
extern CODE_ISR void ISR_TIMER_5();
extern CODE_ISR void ISR_SPI_3();
extern CODE_ISR void ISR_UART_4();
extern CODE_ISR void ISR_UART_5();
extern CODE_ISR void ISR_TIMER_6_AND_DAC();
extern CODE_ISR void ISR_TIMER_7();
extern CODE_ISR void ISR_DMA_2_CH0();
extern CODE_ISR void ISR_DMA_2_CH1();
extern CODE_ISR void ISR_DMA_2_CH2();
extern CODE_ISR void ISR_DMA_2_CH3();
extern CODE_ISR void ISR_DMA_2_CH4();
extern CODE_ISR void ISR_ETHERNET();
extern CODE_ISR void ISR_ETHERNET_WAKEUP();
extern CODE_ISR void ISR_FDCAN_CALIBRATION();
extern CODE_ISR void ISR_ARM_CORTEX_M7_SEND_EVEN();
extern CODE_ISR void ISR_DMA_2_CH5();
extern CODE_ISR void ISR_DMA_2_CH6();
extern CODE_ISR void ISR_DMA_2_CH7();
extern CODE_ISR void ISR_USART_6();
extern CODE_ISR void ISR_I2C_3_EVENT();
extern CODE_ISR void ISR_I2C_3_ERROR();
extern CODE_ISR void ISR_USB_HS_EP1_OUT();
extern CODE_ISR void ISR_USB_HS_EP1_IN();
extern CODE_ISR void ISR_USB_HS_WAKEUP();
extern CODE_ISR void ISR_USB_HS();
extern CODE_ISR void ISR_DCMI();
extern CODE_ISR void ISR_CRYPTO();
extern CODE_ISR void ISR_HASH_RNG();
extern CODE_ISR void ISR_FPU();
extern CODE_ISR void ISR_UART_7();
extern CODE_ISR void ISR_UART_8();
extern CODE_ISR void ISR_SPI_4();
extern CODE_ISR void ISR_SPI_5();
extern CODE_ISR void ISR_SPI_6();
extern CODE_ISR void ISR_SAI_1();
extern CODE_ISR void ISR_LTDC();
extern CODE_ISR void ISR_LTDC_ERROR();
extern CODE_ISR void ISR_DMA2D();
extern CODE_ISR void ISR_SAI_2();
extern CODE_ISR void ISR_QUADSPI();
extern CODE_ISR void ISR_LP_TIMER_1();
extern CODE_ISR void ISR_HDMI_CEC();
extern CODE_ISR void ISR_I2C_4_EVENT();
extern CODE_ISR void ISR_I2C_4_ERROR();
extern CODE_ISR void ISR_SPDIF();
extern CODE_ISR void ISR_USB_FS_EP1_OUT();
extern CODE_ISR void ISR_USB_FS_EP1_IN();
extern CODE_ISR void ISR_USB_FS_WAKEUP();
extern CODE_ISR void ISR_USB_FS();
extern CODE_ISR void ISR_DMAMUX_1_OVERRUN();
extern CODE_ISR void ISR_HR_TIMER_MASTER();
extern CODE_ISR void ISR_HR_TIMER_A();
extern CODE_ISR void ISR_HR_TIMER_B();
extern CODE_ISR void ISR_HR_TIMER_C();
extern CODE_ISR void ISR_HR_TIMER_D();
extern CODE_ISR void ISR_HR_TIMER_E();
extern CODE_ISR void ISR_HR_TIMER_FAULT();
extern CODE_ISR void ISR_DFSDM_FILTER_0();
extern CODE_ISR void ISR_DFSDM_FILTER_1();
extern CODE_ISR void ISR_DFSDM_FILTER_2();
extern CODE_ISR void ISR_DFSDM_FILTER_3();
extern CODE_ISR void ISR_SAI_3();
extern CODE_ISR void ISR_SWPMI();
extern CODE_ISR void ISR_TIMER_15();
extern CODE_ISR void ISR_TIMER_16();
extern CODE_ISR void ISR_TIMER_17();
extern CODE_ISR void ISR_MDIOS_WAKEUP();
extern CODE_ISR void ISR_MDIOS();
extern CODE_ISR void ISR_JPEG();
extern CODE_ISR void ISR_MDMA();
extern CODE_ISR void ISR_SDMMC_2();
extern CODE_ISR void ISR_HSEM();
extern CODE_ISR void ISR_ADC_3();
extern CODE_ISR void ISR_DMAMUX_2_OVERRUN();
extern CODE_ISR void ISR_BDMA_CH0();
extern CODE_ISR void ISR_BDMA_CH1();
extern CODE_ISR void ISR_BDMA_CH2();
extern CODE_ISR void ISR_BDMA_CH3();
extern CODE_ISR void ISR_BDMA_CH4();
extern CODE_ISR void ISR_BDMA_CH5();
extern CODE_ISR void ISR_BDMA_CH6();
extern CODE_ISR void ISR_BDMA_CH7();
extern CODE_ISR void ISR_COMP_1_2();
extern CODE_ISR void ISR_LP_TIMER_2();
extern CODE_ISR void ISR_LP_TIMER_3();
extern CODE_ISR void ISR_LP_TIMER_4();
extern CODE_ISR void ISR_LP_TIMER_5();
extern CODE_ISR void ISR_LP_UART();
extern CODE_ISR void ISR_WWDG_RESET();
extern CODE_ISR void ISR_CRS();
extern CODE_ISR void ISR_RAM_ECC();
extern CODE_ISR void ISR_SAI_4();
extern CODE_ISR void ISR_WAKEUP_PIN_0_TO_5();